function [p,anovatab,stats] = anova1 (c, cluster, keys, displayopt)

% @CORTEX/ANOVA1   analysis of variance on @cortex object spike data
% 
%    [P,ANOVATAB,STATS] = ANOVA1(COBJ,SPIKE,KEYS,DISPLAYOPT)   performs
%    1-way ANOVA on the SPIKE cluster of the COBJ @cortex object. Data
%    grouping and analysis epoch are dermined by KEYS, a @key object
%    containing 2 or more keys. Neuronal responses during trials
%    selected by the keys will be compared. DISPLAYOPT can be 'on'
%    (the default) to display figures containing a standard one-way
%    anova table and a boxplot, or 'off' to omit these displays. See
%    help on Matlab Stats Toolbox's ANOVA1 for an explanation of the
%    output arguments.
% 
% Last modified: 9 Nov 04

if keys.len < 2, error ('the KEYS @key object must contain at least 2 keys'); end
measures = [];
groups   = [];
for k = 1:keys.len
   summa = trialfiringrate (c, cluster, keys(k));
   measures = [measures; summa];
   temp = repmat (k, length(summa), 1);
   groups = [groups; temp];
end
if nargin < 4, displayopt='on'; end
   
[p,anovatab,stats] = anova1 (measures, groups, displayopt);
