function [dense, inst] = density (c, cluster, kwidth, ktype, r, group)

% density.m
% returns a spike density array
%
% Syntax 1: [DENSE, INSTANCES] = DENSITY (COBJ, CLUSTER, WINDOW, TYPE, KEY)
% Syntax 2: [DENSE, INSTANCES] = DENSITY (SPMAT, WINDOW, TYPE)
% Syntax 3: [DENSE, INSTANCES] = DENSITY (FRATES, WINDOW, TYPE);
%
% where
% DENSE:     array of spike density values;
% INSTANCES: number of spike arrays retrieved from COBJ by the RULES criteria
%            i.e., the number of values included in the sum;
% COBJ:      the @cortex object;
% CLUSTER:   the spike number, as encoded in the CORTEX data file;
% SPMAT:     a spike matrix, such as that returned by @cortex/spikematrix.m;
% FRATES:    a 1D vector of values to smooth;
% WINDOW:    window size in ms;
% TYPE:      either 'gauss' or 'boxcar';
% KEY:       one key from a @key object,
%
% The core code for this function is borrowed from Wael's smooth.m (thanks!)
% Andrew Rossi contributed a relevant bug fix (thanks bro)
%
% Last modified: 14 Jul 00

% 13 Jun 99: can pass FRATES as argument
% 22 Feb 00: last line of code fixed according to Andrew's suggestion
% 22 May 00: fixed a small bug: sparr = double (spmat);
%  2 Jun 00: calling filter.m directly (bypassing conv.m);
%            if an odd window is requested, then window -= 1 (instead of +1);
%            if boxcar, increase window by 1 to get same kernel length
% 14 Jul 00: help text fixes

switch nargin
case 6
   spmat = spikematrix (c, cluster, r, group);
case 5
   spmat = spikematrix (c, cluster, r);
otherwise
   error ('wrong number of arguments');
end

[dense, inst] = smoothfr (spmat, kwidth, ktype);
