function [spmat, included_trials] = getspikearray2 (c, cluster, k)

%@cortex/private/getspikearray2.m
%
% returns a spike matrix for a single key.
% 'short' trials are allowed; missing values padded with NaNs
%
% This is a private function, only available
% to @cortex methods
%
% Last modified: 7 Nov 04

% derived from getspikearray on 7 Nov 04

global GiuCoDefaultValues
if isempty (GiuCoDefaultValues), readefs; end

if sum(k.size)>1, error ('single key required'); end
% % included_trials = [];

% first retrieve the subset of trials to be included in the analysis
yestrials = trialset (c, k);
if isempty (yestrials)
   spmat = [];
   return;
end

% allocate tentative matrix space
spmwidth = k.finish - k.start;
spmat = repmat (NaN, length(yestrials), spmwidth);

instance = 0;
cueless = [];

% for each trial in the current cortex object
for trial=yestrials
   times  = c.times (trial, :);
   codes  = c.codes (trial, :);
   spikes = trialcluster (c, cluster, trial);
   datastart = times (find (codes==GiuCoDefaultValues.spikes.encodes(1)));
   datastop  = times (find (codes==GiuCoDefaultValues.spikes.encodes(2)));
   spike_raster = zeros (1, datastop);
   spike_raster (spikes+1) = 1;
   % find cues
   cues_found = intersect (k.cues, codes);
   % warn user if no cue event is found
   if isempty (cues_found), cueless = [cueless trial]; end
   % for each cue event found
   for j=cues_found
      % for each occurrence of the same cue event...
      for cue = times (find (codes==j))
         % find beginning of spm and raster
         startoff = cue + k.start;
         spmidx1 = max (1, datastart - startoff + 1);
         rastidx1 = max (datastart, startoff) + 1;
         % find end of spmat and raster
         endoff = cue + k.finish;
         spmidx2 = spmwidth - max (0, endoff - datastop);
         rastidx2 = min (datastop, endoff);
         % fill spmat row with spikes
         instance = instance + 1;
         included_trials(instance) = trial;
         spmat (instance, spmidx1:spmidx2) = spike_raster (rastidx1:rastidx2);
      end
   end
end
spmat = spmat (1:instance, :);

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
