function [emX, emY, time] = geteyepos (c, trial)

% @CORTEX/PRIVATE/GETEYEPOS   eog positions from a single trial
%
% core code for eye position analysis
%
% Last modified: 5 Nov 04

% based on eyepos.m, last saved on 5 Nov 04

e = c.eog;
first = e.ptr_start (trial);
last  = e.ptr_end   (trial);


samples_per_row = fix ((k.finish-k.start)/sampling);
emX = zeros (length (yestrials), samples_per_row);
emY = zeros (length (yestrials), samples_per_row);
instance = 0;
trials  = [];
cueless = [];
winless = [];
toomanycues = [];

for trial=yestrials
   codes = c.codes(trial, :);
   times = c.times(trial, :);
   % find the first and the last element of the vector 'e.eog_X'
   % that contains the samples for the current trial
   % remember that c.eog_start (trial) == 0 if
   % the current trial does not contain any eog values
   first = e.ptr_start (trial);
   last  = e.ptr_end   (trial);
   samples = last-first+1;
   
   cues = ismember (codes, k.cues);
   switch sum(cues)
      case 0
         cueless = [cueless trial];
         continue;
      case 1
         cuetime = c.times (trial, cues);
      otherwise
         toomanycues = [toomanycues trial];
         cuetime = c.times (trial, min (find (cues)));
   end
   start_time = cuetime + k.start;
   end_time   = cuetime + k.finish;
   start_from_first = start_time - e.start(trial);
   if (start_from_first < 0 | (end_time > (start_time + samples * sampling)))
      winless = [winless trial];
   else
      instance = instance + 1;
      trials = [trials; trial]; 
      findx = first + fix (start_from_first/sampling);
      lindx = findx + samples_per_row-1;
      emX (instance, :) = double(e.X (findx:lindx))'/c.eog2dva;
      emY (instance, :) = double(e.Y (findx:lindx))'/c.eog2dva;
   end
end
emX = emX (1:instance, :);
emY = emY (1:instance, :);
time = k.start:sampling:k.finish-sampling;

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
if ~isempty (winless)
   warning ('could not retrieve all requested trials;');
   disp ('time window exceeds boundaries of the following trial(s)');
   disp (winless);
end
if ~isempty (toomanycues)
   warning ('more than one cue event found in the trials listed below;');
   disp ('only the first one in each trial was considered');
   disp (toomanycues)
end