function c = loadnex(source_filename, eye_on_off, conversion)

%LOADNEX   reads a plexon file
%   first released by NPB on 21 Sep 01
%   Last modified:  1 Aug 02
%                  29 Mar 03
%                  30 Nov 03 

% Cortex encodes (check the Cortex file "encodes.h")
% Modify these as necessary for the way trial info was encoded in your experiment
% TRIAL_START         = 150;
% TRIAL_END           = 151;
% ACTUAL_RESP         = [200 208]; 
% EXPCTD_RESP         = [250 282];
% REPEATNUM           = [99999 99999];
% TTYPENUM            = [99999 99999];
% BLOCKS              = [500 999];
% CONDITIONS          = [1000 4999];

global GiuCoDefaultValues
if isempty (GiuCoDefaultValues), readefs; end
defs = GiuCoDefaultValues.nex;
verbose = GiuCoDefaultValues.general.verbose;
% % fid = openfile ('nexencodes.txt', 'r');
% % while ~feof(fid)
% %    li = fgetl (fid);
% %    linea = trimwhite (li);
% %    if isempty(linea) | linea(1)=='%', continue, end
% %    eval (linea);
% % end

FindX = inline('find(a>=b(1) & a<=b(2))','a','b');

fid = openfile(source_filename, 'r');
[path fname] = fileparts (source_filename);

%version number
c.version = 3;
%the name of the data file
c.name = fname;

c.trials         =  0;
c.cond_no        = [];
c.repeat_no      = [];
c.block_no       = [];
c.eog_rate       = [];
c.KHz_resolution = [];
c.trial_type     = [];
c.exp_response   = [];
c.response       = [];
c.times          = [];
c.codes          = [];
c.cluster_bank   = [];
c.spike_start    = [];
c.spike_end      = [];
c.spikes  = uint32([]);
c.changemode     =  0;
c.rel_trial      = [];
c.eog.dva        =  0;

% % c.eog2dva        =  0;
% % c.eog_start      = [];
% % c.eog_ptr_start  = [];
% % c.eog_ptr_end    = [];
% % c.eog_X    = int16([]);
% % c.eog_Y    = int16([]);
c.spike_channel     = [];

% % c.vretrace          = [];
c.lfp               = [];
% % c.lfp_start         = [];
% % c.lfp_ptr_start     = [];
% % c.lfp_ptr_end       = [];
% % c.lfp_channel       = [];
% % c.lfps        = int16([]);
c.epp.rate           = 0;
c.tch.rate           = 0;

%=====READ FILE INFO=====

[nvar, names, types, header_offsets, freq] = nexf_info(fid,verbose);

%=====READ STROBE/MARKER DATA (i.e. CORTEX ENCODES)=====

marker = find(types==6);
if(isempty(marker) | length(marker)>1)
   error('There is either no STROBED channel or too many markers channels in the file!');
end
if(~strcmp(deblank(names(marker,:)),'STROBED'))
   error('There is no "STROBED" marker variable in the file!');
end

[mtms, mval] = nexf_mark(fid, header_offsets(marker));

%convert timestamps to time in msec (values are rounded to nearest integer)
mtms = round((mtms/freq)*1000);

%find trial start and trial end encodes
tsi = find(mval==defs.trial_start);
if(isempty(tsi))
   error('There are no TRIAL START markers in the datafile!!!');
end
tei = find(mval==defs.trial_end);
if(isempty(tei))
   error('There are no TRIAL END markers in the datafile!!!');
end

%make sure that there is an equal number of trial starts and ends, and that they match in time.
%if this is not the case, fix problems that occur at the beginning or end
while(1)
   if(tei(1)<tsi(1))
      tei(1)=[];
      disp('Found a TRIAL_START problem at the beginning of the file... fixed.');
   else
      break;
   end
end
while(1)
   if(tei(end)<tsi(end))
      tsi(end)=[];
      disp('Found a TRIAL_END problem at the end of the file... fixed.');
   else
      break;
   end
end

if(length(tsi) ~= length(tei))
   error('There is a mismatch in the number of TRIAL_START and TRIAL_END encodes that cannot be fixed.');
else
   for ii=1:length(tsi)
      if(tsi(ii)>tei(ii))
         error(['Trial start and end times do not make sense in trial ' num2str(ii)]);
      end
   end
end

%total number of trials
ntrials = length(tsi);
c.trials = ntrials;

c.cond_no(1:ntrials,1) = NaN;
c.repeat_no(1:ntrials,1) = NaN;
c.block_no(1:ntrials,1) = NaN;
c.eog_rate(1:ntrials,1) = NaN;

%The freq of timestamps in the object.  Note that this is not the original
%freq in the datafile, but the result of the conversion to a msec timescale
%to keep consistent with Cortex conventions
c.KHz_resolution(1:ntrials,1) = 1;

c.trial_type(1:ntrials,1) = NaN;
c.exp_response(1:ntrials,1) = NaN;
c.response(1:ntrials,1) = NaN;

max_codes = max(tei - tsi) + 1;
c.times = zeros(ntrials, max_codes);
c.codes = zeros(ntrials, max_codes);

disp ('retrieving behavioral info');

for ii=1:ntrials
   if((tsi(ii)+1)>(tei(ii)-1))
      continue;
   end
   tempmv = mval(tsi(ii)+1:tei(ii)-1);
   tempmt = mtms(tsi(ii)+1:tei(ii)-1);
   %get conditions
   temp = FindX(tempmv,defs.conditions);
   if(~isempty(temp))
      c.cond_no(ii) = tempmv(temp)' - defs.conditions(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   %get repeat number
   temp = FindX(tempmv,defs.repeat);
   if(~isempty(temp))
      c.repeat_no(ii) = tempmv(temp)' - defs.repeat(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   %get trial type
   temp = FindX(tempmv,defs.trial_type);
   if(~isempty(temp))
      c.trial_type(ii) = tempmv(temp)' - defs.trial_type(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   %get blocks
   temp = FindX(tempmv,defs.blocks);
   if(~isempty(temp))
      c.block_no(ii) = tempmv(temp)' - defs.blocks(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   %get expected and actual responses (these match the cortex codes)
   temp = FindX(tempmv,defs.given_response);
   if(~isempty(temp))
      c.exp_response(ii) = tempmv(temp)' - defs.given_response(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   temp = FindX(tempmv,defs.response_error);
   if(~isempty(temp))
      c.response(ii) = tempmv(temp)' - defs.response_error(1);
      tempmv(temp)=[]; tempmt(temp)=[];
   end
   %fill encode arrays
   if(~isempty(tempmv))
      c.codes(ii,1:length(tempmv)) = tempmv;
      c.times(ii,1:length(tempmv)) = tempmt - mtms(tsi(ii));
   end
end

c.trial_type = zeros(ntrials, 1);

%=====READ SPIKE DATA=====

spkch = find(types==0);
nspks = length(spkch);

if(nspks) 
   disp (['reading ' num2str(nspks) ' spikes']);
   c.cluster_bank = 1:nspks;
   c.spike_start = zeros (ntrials, nspks);
   c.spike_end = zeros (ntrials, nspks);
   %first calculate the total number of possible spikes and allocate spk array
   for ii = c.cluster_bank
      c.spike_channel(ii) = str2num(names(spkch(ii),4:6));
      fseek(fid, header_offsets(spkch(ii))+72, 'bof');
      spk_offset(ii) = fread(fid, 1, 'int32');
      nitems(ii) = fread(fid, 1, 'int32');
   end
   c.spikes = uint32(zeros(sum(nitems), 1));
   %now fill spike-related arrays
   total_spikes = 0;
   spkpos = 1;
   for ii = c.cluster_bank
      disp (num2str(ii));
      fseek(fid, spk_offset(ii), 'bof');
      tspk = fread(fid, [1 nitems(ii)], 'int32');
      %spike times in msec rounded to nearest integer
      tspk = round((tspk/freq)*1000);
      for jj = 1:length(tsi)
         temp = FindX(tspk,[mtms(tsi(jj)) mtms(tei(jj))]);
         if ~isempty(temp)
            c.spike_start(jj,ii) = spkpos;
            spkpos = spkpos + length(temp);
            c.spike_end(jj,ii) = spkpos-1;
            %spike times relative to trial start event
            c.spikes(c.spike_start(jj,ii) : c.spike_end(jj,ii)) = ...
               uint32(tspk(temp) - mtms(tsi(jj)));
            total_spikes = total_spikes + length(temp);
         end
      end            
   end
   c.spikes(total_spikes+1:end)=[];  %clear unfilled indices
   clear nitems;
   clear tspk;
   %make sure that the first spike of a trial is not at time zero.
   %this is necessary because of the way Giuseppe's spike analysis code operates.
   c.spikes(find(c.spikes==0))=1;
else
   c.cluster_bank  = [];
   c.spike_start   = [];
   c.spike_end     = [];
   c.spike_channel = [];
   c.spikes        = uint32([]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % tentatively commented out by GB on Nov 30, 2004
% % %=====READ VERTICAL RETRACE SYNCH DATA=====
% % 
% % disp ('reading vertical retrace')
% % c.vretrace = zeros(ntrials,1);
% % 
% % vr = strmatch('EVENT002', names);
% % if isempty(vr)
% %    disp('This data file does not contain any vertical retrace synch data!');
% % else
% %    fseek(fid, header_offsets(vr)+72, 'bof');
% %    vr_offset = fread(fid, 1, 'int32');
% %    nitems = fread(fid, 1, 'int32');
% %    fseek(fid, vr_offset, 'bof');
% %    vraw = fread(fid, [1 nitems], 'int32');
% %    vraw = round((vraw/freq)*1000);
% %    for ii = 1:ntrials
% %       temp = vraw(FindX(vraw, [mtms(tsi(ii)) mtms(tei(ii))]));
% %       if(~isempty(temp))
% %          c.vretrace(ii, 1:length(temp)) = temp - mtms(tsi(ii));
% %       end
% %    end
% %    clear vraw;
% % end

%=====PREPARE TO READ CONTINUOUS DATA=====
c.eog.start      = zeros(ntrials, 1);
c.eog.ptr_start  = zeros(ntrials, 1);
c.eog.ptr_end    = zeros(ntrials, 1);
c.lfp.start      = zeros(ntrials, 1);
c.lfp.ptr_start  = zeros(ntrials, 1);
c.lfp.ptr_end    = zeros(ntrials, 1);

all_ad = strmatch('AD', names);
if isempty(all_ad)
   warning('This data file does not contain any A/D channels!');
   return;
else
   %Note: all A/D channels have the same number of points at the same time values.
   %Thus, these values need to be read/calculated once only
   [adfreq, ts, fn] = nexf_cont(fid, header_offsets(all_ad(1)));
   %this is the frequency of continuous (e.g., eog, lfp) data with respect
   %to timestamp data; it uses the same convention as in the Cortex menu
   sfreq = 1000/adfreq;
   c.eog_rate(1:ntrials, 1) = sfreq;
   ts = round((ts/freq)*1000);
end

%=====READ EYE POSITION DATA=====

disp ('reading eye positions')
%first 4 letters of eye position data channels
eogchnames = ['AD17';'AD18'];
eog_seekstart(1:ntrials,1) = NaN;
eog_seekend(1:ntrials,1) = NaN;

if nargin > 1
   if(nargin > 2)
      c.eog.dva = conversion;
   else
      c.eog.dva = 204.8;
   end
   for ii = 1:ntrials
      if isempty(eye_on_off)
         eog_seekstart(ii) = mtms(tsi(ii));
         eog_seekend(ii) = mtms(tei(ii));
      else
         if((tsi(ii)+1)>(tei(ii)-1))
            continue;
         end
         tempmv = mval(tsi(ii)+1:tei(ii)-1);
         tempmt = mtms(tsi(ii)+1:tei(ii)-1);
         temp = find(tempmv==eye_on_off(1));
         if ~isempty(temp)
            eog_seekstart(ii) = tempmt(temp);
         else
            error(['No EOG_START found on trial ' num2str(ii)]);
         end
         temp = find(tempmv==eye_on_off(2));
         if ~isempty(temp)
            eog_seekend(ii) = tempmt(temp);
         else
            error(['No EOG_END found on trial ' num2str(ii)]);
         end
      end        
   end
   for ii = 1:size(eogchnames,1)
      varnum = strmatch(eogchnames(ii,:), names);
      if ~isempty(varnum)
         ad = nexf_cont(fid, header_offsets(varnum));
         if ii == 1
            eogindx(1:ceil(sum(eog_seekend-eog_seekstart+(2*sfreq))/sfreq),1) = NaN;
            eogpos = 1;
            for jj = 1:ntrials
               fr = 0;
               for kk = 1:length(fn)
                  if(eog_seekstart(jj) >= ts(kk) & eog_seekstart(jj) <= ts(kk)+(fn(kk)-1)*sfreq)
                     fr = kk;
                     break;
                  end
               end
               if (~fr)
                  warning(['Did not find eye movement points in trial' num2str(jj)]);
               end
               s_ind = round (((eog_seekstart(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
               e_ind = ceil (((eog_seekend(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
               c.eog.start(jj,1) = eog_seekstart(jj) - mtms(tsi(jj));
               c.eog.ptr_start(jj,1) = eogpos;
               eogpos = eogpos + (e_ind - s_ind) + 1;
               c.eog.ptr_end(jj,1) = eogpos-1;
               eogindx(c.eog.ptr_start(jj,1) : c.eog.ptr_end(jj,1)) = [s_ind:e_ind];
            end
            eogindx(find(isnan(eogindx))) = [];
            c.eog.X = int16(ad(eogindx)');
         else
            c.eog.Y = int16(ad(eogindx)');
         end                      
      end
   end
   clear eog_seekstart;
   clear eog_seekend;
   clear eogindx;
   clear ad;
end

numeogs = 0;
for ii = 1:2
   if ~isempty(strmatch(eogchnames(ii,:), names))
      numeogs = numeogs + 1;
   end
end
if(numeogs & nargin==1)
   disp('The data file contained EyePos data which was not processed...');
   disp('...see input arguments to the function.');
end    

%=====READ LOCAL FIELD POTENTIAL (LFP) DATA=====

disp ('reading LFP')
%first 4 letters of LFP data channels
lfpchnames = [ ...
      'AD01';'AD02';'AD03';'AD04';'AD05';'AD06';'AD07';'AD08';...
      'AD09';'AD10';'AD11';'AD12';'AD13';'AD14';'AD15';'AD16'];
%LFPs are processed from trial start to trial end
lfprange   = [defs.trial_start defs.trial_end];
lfpnum     = 0;

%check to see if there are any LFP data to process
totlfp = length(all_ad) - numeogs;    

if totlfp > 0
   lfp_seekstart = mtms(tsi);
   lfp_seekend = mtms(tei);
   for ii = 1:size(lfpchnames,1)
      varnum = strmatch(lfpchnames(ii,:), names);
      if ~isempty(varnum)
         lfpnum = lfpnum + 1;
         c.lfp.channel(lfpnum) = str2num(names(varnum,3:4));
         ad = nexf_cont(fid, header_offsets(varnum));
         if lfpnum == 1
            lfpindx(1:ceil(sum(lfp_seekend-lfp_seekstart+(2*sfreq))/sfreq),1) = NaN;
            lfppos = 1;
            for jj = 1:ntrials
               fr = 0;
               for kk = 1:length(fn)
                  if(lfp_seekstart(jj) >= ts(kk) & lfp_seekstart(jj) <= ts(kk)+(fn(kk)-1)*sfreq)
                     fr = kk;
                     break;
                  end
               end
               if (~fr)
                  warning(['Did not find lfp points in trial' num2str(jj)]);
               end
               s_ind = floor (((lfp_seekstart(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
               e_ind = ceil (((lfp_seekend(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
               c.lfp.start(jj,1) = lfp_seekstart(jj) - mtms(tsi(jj));
               c.lfp.ptr_start(jj,1) = lfppos;
               lfppos = lfppos + (e_ind - s_ind) + 1;
               c.lfp.ptr_end(jj,1) = lfppos-1;
               lfpindx(c.lfp.ptr_start(jj,1) : c.lfp.ptr_end(jj,1)) = [s_ind:e_ind];
            end
            lfpindx(find(isnan(lfpindx))) = [];
            c.lfp.X = int16(zeros(length(lfpindx), totlfp));
         end                      
         c.lfp.X(1:length(lfpindx), lfpnum) = int16(ad(lfpindx));
      end
   end
   clear lfp_seekstart;
   clear lfp_seekend;
   clear lfpindx;
   clear ad;
end

fclose(fid);

s = sprintf('\nData file processed.\n');
disp(s);

return % to main
