function c = loadraw (source_filename, possible_clusters)

% Last modified: 26 Nov 04

%  1 May 02: fixed a bug that excluded spikes occurring at 0 ms from trial start
% 11 Dec 02: temporarily revert to version 2, until plexon stuff is really done
%  4 May 04: just found out that the EPP buffer *precedes* EOGs!!!
%  6 May 04: move up to v. 3 again. Process EPPs
%  5 Nov 04: c.eog.XXX created
%  7 Nov 04: GiuCoDefaultValues implemented
% 26 Nov 04: minibug in spike_channel fixed

global GiuCoDefaultValues
verbose = GiuCoDefaultValues.general.verbose;

i               = 0;
trials          = 0;
max_codes       = 0;
eogs_this_file  = 0;
epps_this_file  = 0;
touch_this_file = 0;
total_spikes    = 0;
head            = zeros (14, 1);
cluster_bank    = [];
epp_warning     = [];
tch_warning     = [];
eye_warning1    = [];
eye_warning2    = [];
eye_warning3    = [];
eye_warning4    = [];

c.version = 3;

f = openfile (source_filename, 'r');
[datapath, name, ext] = fileparts (source_filename);
c.name = [name ext];
if length(c.name) > 30
   c.name(31:end)=[];
   warning ('long file name truncated to 30 chars');
end

if verbose, disp(['Scanning ''', c.name, ''': pass 1...']); end

%%%%%%%%%%%%%%%%%%%
%  P A S S     1  %
%%%%%%%%%%%%%%%%%%%
while 1
   % read the first header entry and use it to test for eof
   % and for file integrity (this value should always be 26)
   header_len = fread (f, 1, 'uint16');
   if feof (f)
      break % end of pass 1, skip to the next stage
   elseif header_len ~= 26
      error ('header size mismatch; corrupted data file?');
   end
   
   trials = trials + 1;
   
   % read the rest of the header
   head ( 2: 9) = fread (f, 8, 'uint16');
   head (10:11) = fread (f, 2, 'uint8' );
   head (12:14) = fread (f, 3, 'uint16');
   
   % check for file integrity
   if head (6) ~= head (7) * 2
      error ('timebuf vs. codebuf mismatch; corrupted data file?')'
   end
   
   % how many events and EyePos samples this trial?
   events_this_trial = head (7)/2;
   eogs_this_trial   = head (8)/4; % number of pairs !!!
   eogs_this_file    = eogs_this_file + eogs_this_trial;
   analogs_this_trial= head (9) / 2; % number of entries !!!
   
   % skip the time stamps
   fseek (f, head(6), 'cof');
   % get the codes
   codes = fread(f, events_this_trial, 'uint16');
   
   % if data are present in EPPbuf, get the channel #s
   if analogs_this_trial
      % read epps as unsigned uint16s in order to read channel#s
      analogs = fread (f, analogs_this_trial, 'uint16');
      % retrieves the channel number from the 4 low-order bits
      channels = bitand (analogs, 15);
      epps_this_file  = epps_this_file  + sum (channels== GiuCoDefaultValues.epp.channels(1));
      touch_this_trial = sum (channels==GiuCoDefaultValues.touch.channels(1));
      if touch_this_trial
         events_this_trial = events_this_trial - sum (codes==GiuCoDefaultValues.touch.encode);
         touch_this_file = touch_this_file + touch_this_trial;
      end
   end
   
   %skip eogs
   fseek (f, head(8), 'cof');
   
   % look for clusters in this trial
   found_clusters = intersect (codes, possible_clusters);
   cluster_bank   = union (cluster_bank, found_clusters);
   spikes_this_trial = sum (ismember  (codes, found_clusters));
   
   total_spikes = total_spikes + spikes_this_trial;
   max_codes = max (max_codes, events_this_trial - spikes_this_trial);
end
%%% end of PASS 1

num_of_clusters = length (cluster_bank);
if verbose
   disp (sprintf ('%d spikes in %d clusters,', total_spikes, num_of_clusters));
   if eogs_this_file
      disp ([num2str(eogs_this_file) ' EOG positions,']);
   end
   if epps_this_file
      disp ([num2str(epps_this_file) ' EPP values,']);
   end
   if touch_this_file
      disp  ([num2str(touch_this_file) ' Touchscreen samples,']);
   end
   disp (['found in ' num2str(trials) ' trials.']);
end

collect_spx = GiuCoDefaultValues.spikes.collect & total_spikes;
collect_eog = GiuCoDefaultValues.eog.collect & eogs_this_file;
collect_epp = GiuCoDefaultValues.epp.collect & epps_this_file;
collect_tch = GiuCoDefaultValues.touch.collect & touch_this_file;

%now the various arrays and variables can be initialized
%for the second pass through the file
%remember that the order of declaration is important:
%only append at the end fields relative to newer versions.

%%%%%%%%%%%%%%
% version  0 %
%%%%%%%%%%%%%%
c.trials         = trials;
c.cond_no        = zeros(trials, 1);
c.repeat_no      = zeros(trials, 1);
c.block_no       = zeros(trials, 1);
c.eog_rate       = zeros(trials, 1);
c.KHz_resolution = zeros(trials, 1);
c.trial_type     = zeros(trials, 1);
c.exp_response   = zeros(trials, 1);
c.response       = zeros(trials, 1);
c.times          = zeros(trials, max_codes);
c.codes          = zeros(trials, max_codes);
c.cluster_bank   = cluster_bank;
c.spike_start    = zeros (trials, num_of_clusters);
c.spike_end      = zeros (trials, num_of_clusters);
c.spikes         = uint32 (zeros (total_spikes, 1));
%%%%%%%%%%%%%%
% version  1 %
%%%%%%%%%%%%%%
c.changemode     = 0;
c.rel_trial      = zeros (trials, 1);
%%%%%%%%%%%%%%
% version  2 %
%%%%%%%%%%%%%%
if collect_eog
   c.eog.dva        = GiuCoDefaultValues.eog.dva;
   c.eog.start      = zeros (trials, 1);
   c.eog.ptr_start  = zeros (trials, 1);
   c.eog.ptr_end    = zeros (trials, 1);
   c.eog.X          = int16 (zeros (eogs_this_file, 1));
   c.eog.Y          = int16 (zeros (eogs_this_file, 1));
   eye_on_off       = GiuCoDefaultValues.eog.encode;
   switch length (eye_on_off)
      case 0
         eye_on  = 0;
         eye_off = 0;
      case 1
         eye_on  = eye_on_off(1);
         eye_off = 0;
      case 2
         eye_on  = eye_on_off(1);
         eye_off = eye_on_off(2);
      otherwise
         error ('invalid encode parameter in [EOG] section of giucode.def');
   end
else
   c.eog.dva = 0;
end
%%%%%%%%%%%%%%%%%%
% version 3 and up
%%%%%%%%%%%%%%%%%%
c.spike_channel     = ones (1, length(cluster_bank));
% % c.vretrace          = [];
% % c.lfp_start         = zeros (trials, 1);
% % c.lfp_ptr_start     = [];
% % c.lfp_ptr_end       = [];
% % c.lfp_channel       = [];
% % c.lfps        = int16([]);
c.lfp = [];

if collect_epp
   c.epp.rate   = GiuCoDefaultValues.epp.rate;
   c.epp.start  = zeros(trials, 1);
   c.epp.ptr    = ones (trials+1, 1);
   c.epp.ch1    = int16(zeros(epps_this_file, 1));
   c.epp.ch2    = int16(zeros(epps_this_file, 1));
else
   c.epp.rate = 0;
end

if collect_tch
   c.tch.rate   = GiuCoDefaultValues.touch.rate;
   c.tch.ptr    = ones  (trials+1, 1);
   c.tch.X      = int16 (zeros(touch_this_file, 1));
   c.tch.Y      = int16 (zeros(touch_this_file, 1));
   c.tch.T      = uint32(zeros(touch_this_file, 1));
else
   c.tch.rate = 0;
end
current_spike_pos = 1;
current_eog_pos = 1;

%%%%%%%%%%%%%%%%%%%
%  P A S S     2  %
%%%%%%%%%%%%%%%%%%%
frewind (f);
if verbose
   disp ('pass 2...');
end
for i=1:trials
   
   head ( 1: 9) = fread (f, 9, 'uint16');
   head (10:11) = fread (f, 2, 'uint8' );
   head (12:14) = fread (f, 3, 'uint16');
   
   c.cond_no(i)        = head  (2) + 1;
   c.repeat_no(i)      = head  (3) + 1;
   c.block_no(i)       = head  (4) + 1;
   c.rel_trial(i)      = head  (5);
   c.eog_rate(i)       = head (10);
   c.KHz_resolution(i) = head (11);
   c.trial_type(i)     = head (12);
   c.exp_response(i)   = head (13);
   c.response(i)       = head (14);
   
   % read in time-stamps and codes
   total_num_of_events = head (7) /2;
   all_times = fread (f, total_num_of_events, 'uint32');
   all_codes = fread (f, total_num_of_events, 'uint16');
   % read in epps (and touchscreen)
   analogs_this_trial = head(9) / 2; % number of individual values
   raw_epps = fread (f, analogs_this_trial, 'int16');
   % read in eogs
   eogs_this_trial = head (8) /4;  % number of pairs !!!
   eogs = fread (f, eogs_this_trial*2, 'int16');
   
   %%%%%%%%%%%%%%%%%%%%%
   % processing spikes %
   %%%%%%%%%%%%%%%%%%%%%
   non_spikes = ones (total_num_of_events, 1);
   for j=1:num_of_clusters
      if ~isempty (all_codes)
         %returns the indices of the current cluster
         temp_cl = find (all_codes == cluster_bank(j));
         if ~isempty (temp_cl)
            %removes this cluster from the non-spike events
            non_spikes (temp_cl) = 0;
            %returns an array containing the time stamps of the corresponding spike
            % 1 May 02: bug found on this line:
            % temp_sp = nonzeros (all_times (temp_cl));
            % spikes occurring at 0ms from start of trial were excluded;
            % a side-effect was a mismatch in dump files
            temp_sp = all_times (temp_cl);
            
            %updates the pointers that will be needed to retrieve the relevant
            %spikes from the HUGE spikes array
            c.spike_start (i, j) = current_spike_pos;
            current_spike_pos    = c.spike_start (i, j) + length (temp_sp);
            c.spike_end   (i, j) = current_spike_pos - 1; 
            c.spikes (c.spike_start (i,j) : c.spike_end (i, j)) = temp_sp;
         end
      end
   end
   
   %%%%%%%%%%%%%%%%%%%%%%%
   % process EPPbuf data %
   %%%%%%%%%%%%%%%%%%%%%%%
   if collect_epp | collect_tch
      % discarding low-order bits to retain data; code derived from
      % Trina's get_ALLdata; there must be a way to avoid those
      % strange additions and subtractions
      all_data = bitshift (raw_epps, -4);
      negs = all_data < 0;
      all_data ( negs) = all_data ( negs) + 2047;
      all_data (~negs) = all_data (~negs) - 2048;
      % convert raw EPPbuf data to unsigned, then read channel number
      % from the 4 low-order bits
      raw_epps ( negs) = raw_epps ( negs) + 65536;
      channels = bitand (raw_epps, 15);
   end
   if collect_epp
      %%%%%%%%%%%%%%%%%%%%%%%
      % process actual EPPs %
      %%%%%%%%%%%%%%%%%%%%%%%
      epp1 = all_data (channels== GiuCoDefaultValues.epp.channels(1));
      epp2 = all_data (channels== GiuCoDefaultValues.epp.channels(2));
      epp_start = all_times (find (all_codes==GiuCoDefaultValues.epp.encode(1)));
      % only one epp collection window is allowed in each trial
      if length(epp_start)>1, epp_warning = [epp_warning i]; end
      c.epp.start(i) = epp_start(1);
      c.epp.ptr(i+1) = c.epp.ptr(i) + length (epp1);
      cr = c.epp.ptr(i):c.epp.ptr(i+1)-1;
      c.epp.ch1 (cr) = epp1;
      c.epp.ch2 (cr) = epp2;
   end
   if collect_tch
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % process TOUCHSCREEN data %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%
      tchX = all_data (channels==GiuCoDefaultValues.touch.channels(1));
      tchY = all_data (channels==GiuCoDefaultValues.touch.channels(2));
      c.tch.ptr(i+1) = c.tch.ptr(i) + length (tchX);
      cr = c.tch.ptr(i):c.tch.ptr(i+1)-1;
      c.tch.X (cr) = tchX;
      c.tch.Y (cr) = tchY;
      touch_idx = find (all_codes == GiuCoDefaultValues.touch.encode);
      if length(touch_idx)==0
         c.tch.T (cr) = 0;
      elseif length(touch_idx) == length (tchX)
         c.tch.T (cr) = all_times (touch_idx);
         non_spikes (touch_idx) = 0;
      else
         tch_warning = [tch_warning i];
         c.tch.T (cr) = 0;
         non_spikes (touch_idx) = 0;
      end
   end
   % % %    % update pointers if analog events exist in the present file but
   % % %    % not in the present trial
   % % %    if collect_epp, c.epp.ptr(i+1) = c.epp.ptr(i); end
   % % %    if collect_tch, c.tch.ptr(i+1) = c.tch.ptr(i); end
   
   % all that is left after removing spikes and touschscreens are
   % behavioral events
   behav_indices       = find (non_spikes);
   num_of_behav_events = length (behav_indices);
   
   if num_of_behav_events
      c.times (i, 1:num_of_behav_events) = all_times (behav_indices)';
      c.codes (i, 1:num_of_behav_events) = all_codes (behav_indices)';
   end
   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % process EOG if necessary %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if collect_eog & eogs_this_trial
      % test eye_on_off parameter
      if eye_on
         eog_start  = all_times (find (all_codes==eye_on));
         switch (length (eog_start))
            case 0
               eye_warning1 = [eye_warning i];
               c.eog.start (i) = NaN;
            case 1
               c.eog.start (i)  = eog_start;
            otherwise
               eye_warning2 = [eye_warning2 i];
               c.eog.start (i) = eog_start(1);
         end
         if eye_off
            eog_end = all_times (find (all_codes==eye_off));
            if isempty (eog_end)
               eye_warning3 = [eye_warning3 i];
            else
               pred_eog_samples = (eog_end-c.eog_start(i))/c.eog_rate(i);
               if pred_eog_samples < (eogs_this_trial-1) | ...
                     pred_eog_samples > (eogs_this_trial+1)
                  eye_warning4 = [eye_warning4 i];
               end
            end
         end
      end
      
      c.eog.ptr_start(i) = current_eog_pos;
      current_eog_pos    = current_eog_pos + eogs_this_trial;
      c.eog.ptr_end  (i) = current_eog_pos - 1;
      
      inz = c.eog.ptr_start (i)';
      fin = c.eog.ptr_end   (i)';
      c.eog.X (inz:fin) = eogs (1:2:eogs_this_trial*2);
      c.eog.Y (inz:fin) = eogs (2:2:eogs_this_trial*2);
   end
end
fclose (f);
%%%%%%%%%%%%%%%%%
% end of pass 2 %
%%%%%%%%%%%%%%%%%

if length(epp_warning)
   s = sprintf ('multiple encodes marking the beginning of EPP collection found in the trials listed below\nAll but the first in each trial were ignored');
   warning (s);
   disp (epp_warning)
end
if length(tch_warning)
   warning  ('number of touchscreen entries in EPPbuf does not match encodes in the trials listed below')
   disp (tch_warning);
end
if length(eye_warning1)
   s = sprintf ('missing EYE_ON encodes in the trials listed below\nno timing information available for those EOG data');
   warning (s);
   disp (eye_warning1);
end
if length(eye_warning2)
   s = sprintf ('multiple EYE_ON encodes found in the trials listed below\nonly first one was considered');
   warning (s);
   disp (eye_warning2);
end
if length (eye_warning3)
   s = sprintf ('missing EYE_OFF encodes in the trials below\nno further measures taken.');
   warning (s);
   disp (eye_warning3);
end
if length (eye_warning4)
   warning ('mismatch in predicted vs. actual number of EOG samples in the trials listed below')
   disp (eye_warning4);
end 

if verbose
   disp (['@cortex object created from file ''', source_filename, '''']);
% %    if eogs_this_file
% %       if c.eog2dva
% %          disp ('Object includes EyePos data.');
% %       else
% %          disp ('EyePos data not loaded.');
% %       end
% %    end
end
return % to main
