function lh = raster (c, cluster, k, tsize);

%RASTER   draws a raster plot on the current axis
%
%   LH = RASTER(C,S,KEY,TSIZE)  creates a raster plot of spike
%   S's firing activity from the C @cortex object, for trials selected
%   and over the epoch specified by KEY, a single key from a @key object.
%
%   TSIZE is a 2-element array that specifies the height of individual
%   tick marks in the raster and the gap between consecutive rows.
%   Default values of .85 and .15 are used if TSIZE is omitted.
%
%   LH is a vector of handles to the lines representing the raster's
%   tick marks.
%
% Last modified: 3 May 04

% 14 Jul 00: numerous changes, including group handling,
%            SPMAT handling, and removal of figure generation.
% 13 Oct 00: bug: xs=r(2).finish;  -->  xf=r(2).finish;
%  3 May 04: function overloaded. See also giucode/raster.m
%            here, just get spmat and call giucode/raster.m

spmat = spikematrix (c, cluster, k);
xlim = [k.start k.finish];

switch nargin
case 3
   lh = raster (spmat, xlim);
case 4
   lh = raster (spmat, xlim, tsize);
otherwise
   error ('wrong number of arguments');
end