function save (c, name)

% @cortex/save.m
% Dumps to disk the entire content of a @cortex object.
% Loading such preprocessed file back to memory is very fast
%
% Syntax 1: SAVE (COBJ, FILENAME);
% Syntax 2: COBJ.SAVE (FILENAME);
%
% Last modified: 10 Nov 04

% 21 apr 99: field changemode added, same name check
% 12 jun 99: version #, rel_trial added
% 10 dec 99: small change in screen output
% 21 may 00: ver.2: eog support + bug fixes
%  1 may 02: ver.3: nex support
% 11 dec 02: revert to ver.2 until plexon stuff is actually done
%  6 nov 04: ver.3: epp, touch support; .nex on hold
% 10 nov 04: bugfix

global GiuCoDefaultValues
if isempty (GiuCoDefaultValues), GiuCoDefaultValues = readefs; end

VERSION = 3;
if strcmp (name, c.name)
   error ('Please use a different file name from the original');
end

f = openfile (name, 'w');

cm = c.changemode;
if cm>1
   cm = cm - 2;
end

% version 0
fwrite (f, '@@@');
fwrite (f, VERSION,             'uint16'); % version number
fwrite (f, c.name                       );
fwrite (f, blanks (30-length(c.name))   );
fwrite (f, c.trials,            'uint16');
fwrite (f, c.cond_no,           'uint16');
fwrite (f, c.repeat_no,         'uint16');
fwrite (f, c.block_no,          'uint16');
fwrite (f, c.eog_rate,          'uchar' );
fwrite (f, c.KHz_resolution,    'uchar' );
fwrite (f, c.trial_type,         'int16');
fwrite (f, c.exp_response,       'int16');
fwrite (f, c.response,          'uint16');
[trials maxevents] = size (c.times);
fwrite (f, maxevents,           'uint16');
fwrite (f, c.times,             'uint32' );
fwrite (f, c.codes,             'uint16');
fwrite (f, length (c.cluster_bank), 'uint16');
fwrite (f, c.cluster_bank,      'uint16');
fwrite (f, c.spike_start,       'uint32');
fwrite (f, c.spike_end,         'uint32');
fwrite (f, double (c.spikes),   'uint32');

% version 1
fwrite (f, cm,                  'uint8' );
fwrite (f, c.rel_trial,         'uint16');

% version 2
% if dva==0 then object does not contain EOG data
% write 0 and move on; otherwise write dva ratio,
% then the rest of the structure
fwrite (f, c.eog.dva,          'float32');
if c.eog.dva
   fwrite (f, c.eog.start,      'uint32');
   fwrite (f, c.eog.ptr_start,  'uint32');
   fwrite (f, c.eog.ptr_end,    'uint32');
   fwrite (f, c.eog.X,           'int16');
   fwrite (f, c.eog.Y,           'int16');
end

%%%%%%%%%%%
% version 3
%%%%%%%%%%%

% needed for multichannel stuff
fwrite (f, c.spike_channel,     'uint16');

% write 0 if object does not contain LFP data
% otherwise 1, then the structure
if isempty(c.lfp)
   fwrite (f, 0, 'uint8');
else
   %%%%%%%%%%%%%%%%%%%%%%%%
   % WARNING!
   % THIS PART WRITTEN
   % BUT NOT TESTED
   %%%%%%%%%%%%%%%%%%%%%%%%
   fwrite (f, 1, 'uint8');
   fwrite (f, c.lfp.start,     'uint32');
   fwrite (f, c.lfp.ptr_start, 'uint32');
   fwrite (f, c.lfp.ptr_end,   'uint32');
   fwrite (f, c.lfp.channel,   'uint32');
   fwrite (f, c.lfp.X,         'int16');
end

% if rate==0 then object does not contain EPP data:
% write 0 and move on; otherwise write rate,
% then the rest of the structure
fwrite (f, c.epp.rate, 'uint16');
if c.epp.rate
   fwrite (f, c.epp.start, 'uint32');
   fwrite (f, c.epp.ptr,   'uint32');
   fwrite (f, c.epp.ch1,   'int16');
   fwrite (f, c.epp.ch2,   'int16');
end

% if rate==0 then object does not contain TOUCH data:
% write 0 and move on; otherwise write rate,
% then the rest of the structure
fwrite (f, c.tch.rate, 'uint16');
if c.tch.rate
   fwrite (f, c.tch.ptr, 'uint32');
   fwrite (f, c.tch.X,   'int16');
   fwrite (f, c.tch.Y,   'int16');
   fwrite (f, c.tch.T,   'uint32');
end

% wrap up
fclose (f);
disp (['@cortex object ''' c.name ''' dumped to:']);
disp (['''' name '''']);
