function count = trialcount (c, varargin)

%TRIALCOUNT   number of trials selected by keys in a @cortex object
%
% COUNT = TRIALCOUNT (COBJ, KEYs1, KEYs2, ...)
% 
% COUNT is a vector containing the number of trials selected from COBJ
% (a @cortex object) by each of the specified KEYs. Note that any
% number of @key objects can be passed, and that each individual key
% from each of the objects will contribute one value to COUNT.
%
% Thus, for example, if ka.len==3 and kb.len==2, then the following to
% calls are equivalent:
%
% tc = trialcount (c, ka(1), ka(2), ka(3), kb(1), kb(2))
% tc = trialcount (c, ka, kb)
%
% Last modified: 6 Nov 04

% Created 1 May 04
% 27 Oct 04: help text fixes; function moved to ./@cortex
%  6 Nov 04: bugfix

count=[];
for pn = 1:length(varargin)
   k = varargin{pn};
   if ~isa (k, 'key'), error ('@key object expected'); end
   for kn = 1:k.len
      tc = length(trialset(c,k(kn)));
      count = [count tc];
   end
end