function [yest, uuni, ugrp] = celltrialset (sp, varargin)

%@SPIX/CELLTRIALSET  finds trial indices in a @spix object
%   
%   same as @SPIX/TRIALSET, except:
%
%   TSET = CELLTRIALSET(...) returns TSET, an M-by-N cell array in
%   which each cell contains the trial indices for one unit; M is the
%   number of retrieved units, and N is the number of retrieved keys,
%   combos or levels.
%
%   [TSET,UNITS,CRIT] = CELLTRIALSET(...) also returns the list of
%   retrieved UNITS, and CRIT, the list of retrieved keys, combos, or
%   levels. It follows that:
%   length(UNITS)==size(TSET,1), and length(CRIT)==size(TSET,2)
%
%Modified: 27 Nov 04

% 27 Apr 03, by Giuseppe Bertini
% 27 Nov 04: help text revised

[yestrials, uni, grp] = trialset (sp, varargin{:});

uuni = unique(uni);
ugrp = unique(grp);

for ce = 1:length(uuni)
   for gr = 1:length(ugrp)
      yest {ce, gr} = yestrials ((uni==uuni(ce)) & (grp==ugrp(gr)));
   end
end