function a = subsref (o, s)

% @spix/subsref.m
%
% Subscript handling routine for the @spix class.
% The user of the class does not need to know how it works.
%
% Last modified: 21 Aug 03

% Last unnoted changes: 10 Aug 02
% 23 Dec 02: small bug fixed
% 08 May 03: bugs
% 30 May 03: added sp.key.pri .siz .len
% 10 Aug 03: added sp.key.lab, sp.key([]).lab, combo
% 18 Aug 03: fixed bugs in 3-sub key processing
% 21 Aug 03: added sp.name sp.label sp.key.lis

% how many subscripting levels?
switch length (s)
   case 1
      % "global" properties, e.g.,
      % s.units; s.nunits; s.factors; s.info
      a = onesub (o, s, nargout);
   case 2
      % two kinds:
      % s.units.labels; s.factors.count
      % s.find(); s.add()
      a = twosubs (o, s);
   case 3
      % s.units(x).labels
      %
      a = threesubs (o, s);
   case 4
      a = foursubs (o, s);
   otherwise
      error ('too many subscripting levels')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
function a = onesub (o, s, needout)
%%%%%%%%%%%%%%%%%%%%%%%%%%
a = [];
switch s.type
   case '.'
      switch minifield (s.subs)
         case 'ver'
            a = o.verbose;
         case 'inf'
            disp (['this spix, named ' o.name ', contains:']);
            disp (sprintf ('%4d units', o.nunits));
            disp (sprintf ('%4d subset(s) of units', o.nsubs));
            disp (sprintf ('%4d keys', o.k.len));
            disp (sprintf ('%4d group(s) of keys', o.ngroups));
            disp (sprintf ('%4d combination(s) of keys', o.ncombos));
            disp (sprintf ('%4d factor(s)', o.nfactors));
         case 'uni'
            disp ([num2str(o.nunits) ' total units in the spix']);
            a = o.info;
         case 'inc'
            disp ([num2str(length(o.inclunits)) ' included units in the spix']);
            a = o.info(o.inclunits);
         case 'sub'
            disp ([num2str(o.nsubs) ' subsets of units defined in this spix']);
            a = o.subsets;
         case 'key'
            disp ([num2str(o.k.len) ' total keys in the spix']);
            a = o.k;
         case 'ink'
            disp ([num2str(length(o.inkeys)) ' included keys in the spix']);
            a = o.inkeys;
         case 'fac'
            disp ([num2str(o.nfactors) ' factors defined in the spix']);
            a = o.factors;
         case 'gro'
            disp ([num2str(o.ngroups) ' groups of keys defined in this spix']);
            a = o.groups;
         case 'com'
            disp ([num2str(o.ncombos) ' combos of keys defined in this spix']);
            a = o.combos;
         case {'lab' 'nam'}
            a = o.name;
         otherwise
            error ('bad syntax');
      end
   otherwise
      error ('bad syntax');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a = twosubs (o, s)
%%%%%%%%%%%%%%%%%%%%%%%%%%%
if s(1).type == '.' & s(2).type == '.'
   switch minifield (s(1).subs)
      case 'uni'
         a = stdreturn (o.info, s(2).subs);
      case 'inc'
         a = stdreturn (o.info, s(2).subs, o.inclunits);
      case 'key'
         switch minifield (s(2).subs)
            case 'pri'
               print (o.k);
               a=[];
            case {'len' 'cou'}
               a = o.k.len;
            case 'siz'
               a = o.k.siz;
            case 'inc'
               a = o.inkeys;
            case 'lab'
               a = o.k.lab;
            case 'lis'
               a = 1:o.k.len;
            otherwise
               error ('bad syntax');
         end
      case 'ink'
         switch minifield (s(2).subs)
            case {'len' 'cou'}
               a = length(o.inkeys);
            case 'lis'
               a = o.inkeys;
            otherwise
               error ('bad syntax');
         end
      case 'fac'
         a = stdreturn (o.factors, s(2).subs);
      case 'sub'
         a = stdreturn (o.subsets, s(2).subs);
      case 'gro'
         a = stdreturn (o.groups, s(2).subs);
      case 'com'
         a = stdreturn (o.combos, s(2).subs);
      otherwise
         error ('bad syntax');
   end
   
elseif s(1).type == '.' & s(2).type == '()'
   switch minifield (s(1).subs)
      case 'fin' % find
         tro = s(2).subs{1};
         found = false;
         if ~isstr (tro), error ('bad syntax'), end
               
         a = lbl2idx (o.info, tro);
         if a
            disp ([tro ' is a unit']);
            found = true;
%             break;
         end
         a = lbl2idx (o.subsets, tro);
         if a
            disp ([tro ' is a subset']);
            found = true;
%             break;
         end
         a = lbl2idx (o.factors, tro);
         if a
            disp ([tro ' is a factor']);
            found = true;
%             break;
         end
         for fid = 1:o.nfactors
            a = lbl2idx (o.factors(fid).levels, tro);
            if a
               disp ([tro ' is a level of factor ' o.factors(fid).label]);
               found = true;
               break
            end
         end
         if ~found
            disp ([tro ' not found']);
            a = [];
         end
      case 'add'
         a = add (o, s(2).subs{1});
      case 'sub'
         subs = parsegrouping (o.subsets, s(2).subs{1});
         a = o.subsets(subs);
      case 'fac'
         subs = parsegrouping (o.factors, s(2).subs{1});
         a = o.factors(subs);
      case 'gro'
         gro = parsegrouping (o.groups, s(2).subs{1});
         a = o.groups(gro);
      case 'com'
         com = parsegrouping (o.combos, s(2).subs{1});
         a = o.combos(com);
      case 'uni'
         uni = parsegrouping (o.info, s(2).subs{1});
         a = o.info(uni);
      case 'inc'
         uni = parsegrouping (o.info, s(2).subs{1});
         siuni = intersect (uni, o.inclunits);
         a = o.info(siuni);
      case 'key'
         a = o.k(s(2).subs{1});
      otherwise
         error ('bad syntax');
   end
else
   error ('bad syntax');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a = threesubs (o, s)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if s(1).type=='.' & s(2).type == '()' & s(3).type=='.'
   switch minifield (s(1).subs)
      case 'uni'
         units = parsegrouping (o.info, s(2).subs{1});
         a = stdreturn (o.info, s(3).subs, units);
      case 'fac'
         facts = parsegrouping (o.factors, s(2).subs{1});
         a = stdreturn (o.factors, s(3).subs, facts);
      case 'sub'
         subs = parsegrouping (o.subsets, s(2).subs{1});
         a = stdreturn (o.subsets, s(3).subs, subs);
      case 'gro'
         gro = parsegrouping (o.groups, s(2).subs{1});
         a = stdreturn (o.groups, s(3).subs, gro);
      case 'com'
         com = parsegrouping (o.combos, s(2).subs{1});
         a = stdreturn (o.combos, s(3).subs, com);
      case 'key'
         keys = parsegrouping (o.k, s(2).subs{1});
         switch minifield (s(3).subs)
            case 'lab'
               for i=1:length(keys)
                  a{i} = o.k(keys(i)).lab;
               end
            case 'lis'
               a = keys;
            case 'cou'
               a = length(keys);
            otherwise
               error ('bad syntax');
         end
      otherwise
         error ('bad syntax');
   end
else
   error ('bad syntax');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
function a = foursubs (o, s)
%%%%%%%%%%%%%%%%%%%%%%%%%%
if s(1).type=='.' & ...
      s(2).type=='()' & ...
      s(3).type=='.' & ...
      s(4).type=='.'
   switch minifield (s(1).subs)
      case 'fac'
         facts = parsegrouping (o.factors, s(2).subs{1});
         if length(facts)>1, error ('only one factor please'); end
         if ~strcmp (minifield (s(3).subs), 'lev'), error ('bad syntax'); end
         a = stdreturn (o.factors(facts).levels, s(4).subs);
      case 'sub'
         subs = parsegrouping (o.subsets, s(2).subs{1});
         if length(subs)>1, error ('only one subset please'); end
         if ~strcmp (minifield (s(3).subs), 'uni'), error ('bad syntax'); end
         uni = o.subsets(subs).units;
         a = stdreturn (o.info(uni), s(4).subs);
         
      otherwise
         error ('bad syntax');
   end
elseif s(1).type=='.'  & ...
       s(2).type=='()' & ...
       s(3).type=='.'  & ...
       s(4).type=='()'
   if ~strcmp (minifield (s(1).subs), 'fac') | ...
         ~strcmp (minifield (s(3).subs), 'lev')
      error ('bad syntax');
   end
   fact = parsegrouping (o.factors, s(2).subs{1});
   if ~isnumscalar (fact), error ('only one factor please'); end
   levs = parsegrouping (o.factors(fact).levels, s(4).subs{1});
   a = o.factors(fact).levels(levs);
else
   error ('bad syntax');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a = stdreturn (stru, cosa, quali, s)

if nargin<3, quali = 1:length(stru); end
switch minifield (cosa)
   case 'lab'
      if length(quali)==1
         a = stru(quali).label;
      else
         a = {};
         for id = 1:length(quali)
            a{id} = stru(quali(id)).label;
         end
      end
   case 'lis'
      a = quali;
   case 'cou'
      a = length (quali);
   case 'lev'
      a = stru(quali).levels;
   case 'uni'
      a = stru(quali).units;
   case 'key'
      a = stru(quali).keys;
   otherwise
      error ('bad syntax');
end