function varargout = cortview(varargin)
% CORTVIEW  simple GUI for @cortex object browsing
%
%   CORTVIEW(CO) creates a GUI figure that shows header values, behavioral
%   encodes, and spikes for each trial in the CO @cortex object.
%
%   CORTVIEW(CO,K) also loads the K @key object. Click on one of the key
%   labels in the bottom left list box in order to restrict browsing to
%   specific subsets of trials.
%
%   Note that you can also invoke CORTVIEW using the two following
%   @cortex dot-property syntax forms:
%   >> CO.VIEW;
%   >> CO.VIEW(K);
%
% Last modified: 3 Dec 04

% Edit the above text to modify the response to help cortview

% Last Modified by GUIDE v2.5 03-Dec-2004 08:00:49

% Begin initialization code - DO NOT EDIT
gui_Singleton = 0;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @cortview_OpeningFcn, ...
                   'gui_OutputFcn',  @cortview_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before cortview is made visible.
function cortview_OpeningFcn(hObject, eventdata, handles, varargin)

kwarn = {};
kl = {'All trials'};
handles.c = varargin{1};
if ~isa (handles.c, 'cortex'), error ('@cortex object needed'); end
if length(varargin)>1
   k = varargin{2};
   if ~isa (k, 'key'), error ('@key object needed'); end
   for i=1:k.len
      ts = trialset (handles.c, k(i));
      if isempty(ts)
         kwarn = [kwarn; {k(i).lab}];
      else
         kl = [kl {k(i).lab}];
      end
   end
   set (handles.lstKeys, 'string', kl);
   handles.k = k;
   if ~isempty (kwarn)
      warning ('The following keys were discarded because they do not match any trials:');
      disp (kwarn);
   end
else
   set (handles.lstKeys, 'string', 'No keys loaded');
end
set (handles.figure1, 'name', handles.c.name);
handles.vtrials = 1:handles.c.trials; % current valid trials array
handles.vtridx = 1; % current valid trials index
set (handles.tTrial, 'string', 1);
UpdateScreen (handles);

% Choose default command line output for cortview
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% --- Outputs from this function are returned to the command line.
function varargout = cortview_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes on selection change in lstCodes.
function lstCodes_Callback(hObject, eventdata, handles)

% --- Executes on selection change in lstSpikes.
function lstSpikes_Callback(hObject, eventdata, handles)

% --- Executes on button press in cmdFirst.
function cmdFirst_Callback(hObject, eventdata, handles)
set (handles.tTrial, 'string', handles.vtrials(1));
handles.vtridx = 1;
guidata(hObject, handles);
UpdateScreen (handles);

% --- Executes on button press in cmdPrev.
function cmdPrev_Callback(hObject, eventdata, handles)
idx = max (1, handles.vtridx - 1);
set (handles.tTrial, 'string', handles.vtrials (idx));
handles.vtridx = idx;
guidata(hObject, handles);
UpdateScreen (handles);

% --- Executes on button press in cmdNext.
function cmdNext_Callback(hObject, eventdata, handles)
idx = min (length(handles.vtrials), handles.vtridx + 1);
set (handles.tTrial, 'string', handles.vtrials (idx));
handles.vtridx = idx;
guidata(hObject, handles);
UpdateScreen (handles);

% --- Executes on button press in cmdLast.
function cmdLast_Callback(hObject, eventdata, handles)
set (handles.tTrial, 'string', handles.vtrials(end));
handles.vtridx = length(handles.vtrials);
guidata(hObject, handles);
UpdateScreen (handles);

function tTrial_Callback(hObject, eventdata, handles)
ct = max (1, min(uivalue(hObject), handles.c.tri));
ids = find (handles.vtrials <= ct); % smaller id's
if isempty (ids)
   idb = find (handles.vtrials > ct);
   handles.vtridx = min (idb);
else
   handles.vtridx = max (ids);
end
nt = handles.vtrials(handles.vtridx);
set (handles.tTrial, 'string', nt);
guidata(hObject, handles);
UpdateScreen (handles);

function UpdateScreen (handles)
ct = uivalue (handles.tTrial);
c  = handles.c;
set (handles.tRel, 'string', c.rel(ct));
set (handles.tCnd, 'string', c.con(ct));
set (handles.tBlo, 'string', c.blo(ct));
set (handles.tRep, 'string', c.rep(ct));
set (handles.tEOG, 'string', c.eog(ct));
set (handles.tKHz, 'string', c.khz(ct));
set (handles.tTyp, 'string', c.typ(ct));
set (handles.tGiv, 'string', c.giv(ct));
set (handles.tRes, 'string', c.res(ct));
set (handles.tSpikes, 'string', length(trialspikes(c, ct)));

% set up lstCodes
% the weird formatting technique is due to the fact that
% num2str chops off trailing columns of blanks
idx = find (c.codes(ct,:));
nix = length (idx);
iii = num2str ((1:nix)', 'o%3d');
tmp = c.times (ct,idx)';
ttt = num2str (tmp, 'o%8d');
tmp = [0; diff(tmp)];
ddd = num2str (tmp, 'o%6d');
ccc = num2str (c.codes (ct, idx)', 'o%6d');
sss = [iii(:,2:end), ttt(:,2:end), ddd(:,2:end), ccc(:,2:end)];
head = '  # elapsed  diff  code';
set (handles.lstCodes, 'string', [head;sss]);

% set up lstSpikes
[ts cl] = trialspikes (c, ct);
clu = num2str (cl); [hi w1] = size(clu);
tim = num2str (ts); w2 = size(tim,2);
spc = repmat(' ', hi, 2);
sss = [clu, spc, tim];
if isempty (sss)
   set (handles.lstSpikes, 'string', '#    ms');
else
   head = repmat (' ', 1, size(sss,2));
   head (w1) = '#';
   head (end-1:end) = 'ms';
   set (handles.lstSpikes, 'string', [head; sss]);
end

% --- Executes on selection change in lstKeys.
function lstKeys_Callback(hObject, eventdata, handles)
ck = get (hObject, 'value');
if ck==1
   handles.vtrials = 1:handles.c.trials;
else
   kl = get(hObject, 'string');
   handles.vtrials = trialset (handles.c, handles.k(kl{ck}));
end
handles.vtridx = 1;
set (handles.tTrial, 'string', handles.vtrials(1));
guidata(hObject, handles);
UpdateScreen(handles);