function lh = raster (spmat, xlim, tsize);

%RASTER   draws a raster plot on the current axis
%
%   LH = RASTER(SPMAT,XLIM,TSIZE)  creates a raster plot of a neuron's
%   firing activity, based on the SPMAT spikematrix.
%
%   XLIM specifies the time in ms corresponding to the first column in
%   SPMAT, relative to the cue event. This is needed to properly set
%   the axis' 'xlim' property. If omitted, XLIM defaults to 1.
%
%   TSIZE is a 2-element array that specifies the height of individual
%   tick marks in the raster and the gap between consecutive rows.
%   Default values of .85 and .15 are used if TSIZE is omitted.
%
%   LH is a vector of handles to the lines representing the raster's
%   tick marks.
%
% Last modified: 27 Oct 04

% 14 Jul 00: numerous changes, including group handling,
%            SPMAT handling, and removal of figure generation.
% 13 Oct 00: bug: xs=r(2).finish;  -->  xf=r(2).finish;
%  3 May 04: function overloaded. See also @cortex/raster.m
% 27 Oct 04: bug in argument checking code; cosmetics

errifnotspikematrix(spmat);
[rows, cols] = size (spmat);
switch nargin
case 1
   xlim = [1, cols];
   tsize = [.85 .15];
case 2
   if isnumscalar (xlim) % then this is really XLIM
      xlim(2) = xlim(1) + cols;
      tsize = [.85 .15];
   elseif prod(size(xlim))==2 % then this must be TSIZE
      tsize = xlim;
      xlim = [1, cols];
   else
      error ('2nd argument is either XLIM, a scalar, or TSIZE, a 2-element vector');
   end
case 3
   xlim(2) = xlim(1) + cols;
otherwise
   error ('wrong number of arguments');
end

[ly, lx] = find (spmat);
ly = [ly' + tsize(2); ly' + tsize(1)];
lx = [lx' + xlim(1); lx' + xlim(1)];

lh = line (lx, ly);
axis ij;
set (gca, 'ylim', [1 rows+1], 'xlim', xlim, 'tickdir', 'out');
set (lh, 'color', 'black');